@@ -3,7 +3,7 @@
 
 /*
    Copyright (c) 2000, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2019, MariaDB
+   Copyright (c) 2009, 2021, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -261,6 +261,12 @@ public:
   Item_func_concat(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
   String *val_str(String *);
   bool fix_length_and_dec();
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   const char *func_name() const { return "concat"; }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_concat>(thd, this); }
@@ -281,7 +287,18 @@ public:
    :Item_func_concat(thd, a, b)
   { }
   String *val_str(String *);
-  const char *func_name() const { return "concat_operator_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
+  void print(String *str, enum_query_type query_type)
+  {
+    if (query_type & QT_FOR_FRM)
+    {
+      // 10.3 downgrade compatibility for FRM
+      str->append(STRING_WITH_LEN("concat_operator_oracle"));
+    }
+    else
+      print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   Item *get_copy(THD *thd)
   {
     return get_item_copy<Item_func_concat_operator_oracle>(thd, this);
@@ -336,12 +353,19 @@ public:
 class Item_func_replace :public Item_str_func
 {
   String tmp_value,tmp_value2;
+protected:
+  String *val_str_internal(String *str, bool null_to_empty);
 public:
   Item_func_replace(THD *thd, Item *org, Item *find, Item *replace):
     Item_str_func(thd, org, find, replace) {}
-  String *val_str(String *to) { return val_str_internal(to, NULL); };
+  String *val_str(String *to) { return val_str_internal(to, false); };
   bool fix_length_and_dec();
-  String *val_str_internal(String *str, String *empty_string_for_null);
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   const char *func_name() const { return "replace"; }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_replace>(thd, this); }
@@ -354,8 +378,19 @@ class Item_func_replace_oracle :public Item_func_replace
 public:
   Item_func_replace_oracle(THD *thd, Item *org, Item *find, Item *replace):
     Item_func_replace(thd, org, find, replace) {}
-  String *val_str(String *to) { return val_str_internal(to, &tmp_emtpystr); };
-  const char *func_name() const { return "replace_oracle"; }
+  String *val_str(String *to) { return val_str_internal(to, true); };
+  const Schema *schema() const { return &oracle_schema_ref; }
+  void print(String *str, enum_query_type query_type)
+  {
+    if (query_type & QT_FOR_FRM)
+    {
+      // 10.3 downgrade compatibility for FRM
+      str->append(STRING_WITH_LEN("replace_oracle"));
+    }
+    else
+      print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_replace_oracle>(thd, this); }
 };
@@ -367,25 +402,55 @@ class Item_func_regexp_replace :public Item_str_func
   bool append_replacement(String *str,
                           const LEX_CSTRING *source,
                           const LEX_CSTRING *replace);
+protected:
+  String *val_str_internal(String *str, bool null_to_empty);
 public:
   Item_func_regexp_replace(THD *thd, Item *a, Item *b, Item *c):
     Item_str_func(thd, a, b, c)
     {}
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   void cleanup()
   {
-    DBUG_ENTER("Item_func_regex::cleanup");
+    DBUG_ENTER("Item_func_regexp_replace::cleanup");
     Item_str_func::cleanup();
     re.cleanup();
     DBUG_VOID_RETURN;
   }
-  String *val_str(String *str);
-  bool fix_fields(THD *thd, Item **ref);
+  String *val_str(String *str)
+  {
+    return val_str_internal(str, false);
+  }
   bool fix_length_and_dec();
   const char *func_name() const { return "regexp_replace"; }
   Item *get_copy(THD *thd) { return 0;}
 };
 
 
+class Item_func_regexp_replace_oracle: public Item_func_regexp_replace
+{
+public:
+  Item_func_regexp_replace_oracle(THD *thd, Item *a, Item *b, Item *c)
+   :Item_func_regexp_replace(thd, a, b, c)
+  {}
+  const Schema *schema() const { return &oracle_schema_ref; }
+  bool fix_length_and_dec()
+  {
+    bool rc= Item_func_regexp_replace::fix_length_and_dec();
+    maybe_null= true; // Empty result is converted to NULL
+    return rc;
+  }
+  String *val_str(String *str)
+  {
+    return val_str_internal(str, true);
+  }
+};
+
+
 class Item_func_regexp_substr :public Item_str_func
 {
   Regexp_processor_pcre re;
@@ -395,13 +460,12 @@ public:
     {}
   void cleanup()
   {
-    DBUG_ENTER("Item_func_regex::cleanup");
+    DBUG_ENTER("Item_func_regexp_substr::cleanup");
     Item_str_func::cleanup();
     re.cleanup();
     DBUG_VOID_RETURN;
   }
   String *val_str(String *str);
-  bool fix_fields(THD *thd, Item **ref);
   bool fix_length_and_dec();
   const char *func_name() const { return "regexp_substr"; }
   Item *get_copy(THD *thd) { return 0; }
@@ -492,8 +556,16 @@ public:
   Item_func_substr(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
   Item_func_substr(THD *thd, Item *a, Item *b, Item *c):
     Item_str_func(thd, a, b, c) {}
+  Item_func_substr(THD *thd, List<Item> &list)
+    :Item_str_func(thd, list) {}
   String *val_str(String *);
   bool fix_length_and_dec();
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   const char *func_name() const { return "substr"; }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_substr>(thd, this); }
@@ -511,13 +583,26 @@ public:
     Item_func_substr(thd, a, b) {}
   Item_func_substr_oracle(THD *thd, Item *a, Item *b, Item *c):
     Item_func_substr(thd, a, b, c) {}
+  Item_func_substr_oracle(THD *thd, List<Item> &list)
+    :Item_func_substr(thd, list) {}
   bool fix_length_and_dec()
   {
     bool res= Item_func_substr::fix_length_and_dec();
     maybe_null= true;
     return res;
   }
-  const char *func_name() const { return "substr_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
+  void print(String *str, enum_query_type query_type)
+  {
+    if (query_type & QT_FOR_FRM)
+    {
+      // 10.3 downgrade compatibility for FRM
+      str->append(STRING_WITH_LEN("substr_oracle"));
+    }
+    else
+      print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_substr_oracle>(thd, this); }
 };
@@ -560,13 +645,13 @@ protected:
   {
     return trimmed_value(res, 0, res->length());
   }
-  virtual const char *func_name_ext() const { return ""; }
 public:
   Item_func_trim(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
   Item_func_trim(THD *thd, Item *a): Item_str_func(thd, a) {}
   Sql_mode_dependency value_depends_on_sql_mode() const;
   String *val_str(String *);
   bool fix_length_and_dec();
+  const Schema *schema() const { return &mariadb_schema; }
   const char *func_name() const { return "trim"; }
   void print(String *str, enum_query_type query_type);
   virtual const char *mode_name() const { return "both"; }
@@ -580,12 +665,11 @@ class Item_func_trim_oracle :public Item_func_trim
 protected:
   String *make_empty_result(String *str)
   { null_value= 1; return NULL; }
-  const char *func_name_ext() const { return "_oracle"; }
 public:
   Item_func_trim_oracle(THD *thd, Item *a, Item *b):
     Item_func_trim(thd, a, b) {}
   Item_func_trim_oracle(THD *thd, Item *a): Item_func_trim(thd, a) {}
-  const char *func_name() const { return "trim_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
   bool fix_length_and_dec()
   {
     bool res= Item_func_trim::fix_length_and_dec();
@@ -607,6 +691,7 @@ public:
     return Item_func::value_depends_on_sql_mode();
   }
   String *val_str(String *);
+  const Schema *schema() const { return &mariadb_schema; }
   const char *func_name() const { return "ltrim"; }
   const char *mode_name() const { return "leading"; }
   Item *get_copy(THD *thd)
@@ -619,12 +704,11 @@ class Item_func_ltrim_oracle :public Item_func_ltrim
 protected:
   String *make_empty_result(String *str)
   { null_value= 1; return NULL; }
-  const char *func_name_ext() const { return "_oracle"; }
 public:
   Item_func_ltrim_oracle(THD *thd, Item *a, Item *b):
     Item_func_ltrim(thd, a, b) {}
   Item_func_ltrim_oracle(THD *thd, Item *a): Item_func_ltrim(thd, a) {}
-  const char *func_name() const { return "ltrim_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
   bool fix_length_and_dec()
   {
     bool res= Item_func_ltrim::fix_length_and_dec();
@@ -642,6 +726,7 @@ public:
   Item_func_rtrim(THD *thd, Item *a, Item *b): Item_func_trim(thd, a, b) {}
   Item_func_rtrim(THD *thd, Item *a): Item_func_trim(thd, a) {}
   String *val_str(String *);
+  const Schema *schema() const { return &mariadb_schema; }
   const char *func_name() const { return "rtrim"; }
   const char *mode_name() const { return "trailing"; }
   Item *get_copy(THD *thd)
@@ -654,12 +739,11 @@ class Item_func_rtrim_oracle :public Item_func_rtrim
 protected:
   String *make_empty_result(String *str)
   { null_value= 1; return NULL; }
-  const char *func_name_ext() const { return "_oracle"; }
 public:
   Item_func_rtrim_oracle(THD *thd, Item *a, Item *b):
     Item_func_rtrim(thd, a, b) {}
   Item_func_rtrim_oracle(THD *thd, Item *a): Item_func_rtrim(thd, a) {}
-  const char *func_name() const { return "rtrim_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
   bool fix_length_and_dec()
   {
     bool res= Item_func_rtrim::fix_length_and_dec();
@@ -821,6 +905,12 @@ class Item_func_decode :public Item_func_encode
 {
 public:
   Item_func_decode(THD *thd, Item *a, Item *seed_arg): Item_func_encode(thd, a, seed_arg) {}
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   const char *func_name() const { return "decode"; }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_decode>(thd, this); }
@@ -1126,6 +1216,8 @@ public:
     Item_str_func(thd, arg1, arg2, arg3) {}
   Item_func_pad(THD *thd, Item *arg1, Item *arg2):
     Item_str_func(thd, arg1, arg2) {}
+  Item_func_pad(THD *thd, List<Item> &list):
+    Item_str_func(thd,list) {}
   bool fix_length_and_dec();
 };
 
@@ -1137,7 +1229,15 @@ public:
     Item_func_pad(thd, arg1, arg2, arg3) {}
   Item_func_rpad(THD *thd, Item *arg1, Item *arg2):
     Item_func_pad(thd, arg1, arg2) {}
+  Item_func_rpad(THD *thd, List<Item> &list):
+    Item_func_pad(thd,list) {}
   String *val_str(String *);
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   const char *func_name() const { return "rpad"; }
   Sql_mode_dependency value_depends_on_sql_mode() const;
   Item *get_copy(THD *thd)
@@ -1154,13 +1254,26 @@ public:
     Item_func_rpad(thd, arg1, arg2, arg3) {}
   Item_func_rpad_oracle(THD *thd, Item *arg1, Item *arg2):
     Item_func_rpad(thd, arg1, arg2) {}
+  Item_func_rpad_oracle(THD *thd, List<Item> &list):
+    Item_func_rpad(thd,list) {}
   bool fix_length_and_dec()
   {
     bool res= Item_func_rpad::fix_length_and_dec();
     maybe_null= true;
     return res;
   }
-  const char *func_name() const { return "rpad_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
+  void print(String *str, enum_query_type query_type)
+  {
+    if (query_type & QT_FOR_FRM)
+    {
+      // 10.3 downgrade compatibility for FRM
+      str->append(STRING_WITH_LEN("rpad_oracle"));
+    }
+    else
+      print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_rpad_oracle>(thd, this); }
 };
@@ -1173,7 +1286,15 @@ public:
     Item_func_pad(thd, arg1, arg2, arg3) {}
   Item_func_lpad(THD *thd, Item *arg1, Item *arg2):
     Item_func_pad(thd, arg1, arg2) {}
+  Item_func_lpad(THD *thd, List<Item> &list):
+    Item_func_pad(thd,list) {}
   String *val_str(String *);
+  const Schema *schema() const { return &mariadb_schema; }
+  void print(String *str, enum_query_type query_type)
+  {
+    print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   const char *func_name() const { return "lpad"; }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_lpad>(thd, this); }
@@ -1189,13 +1310,26 @@ public:
     Item_func_lpad(thd, arg1, arg2, arg3) {}
   Item_func_lpad_oracle(THD *thd, Item *arg1, Item *arg2):
     Item_func_lpad(thd, arg1, arg2) {}
+  Item_func_lpad_oracle(THD *thd, List<Item> &list):
+    Item_func_lpad(thd,list) {}
   bool fix_length_and_dec()
   {
     bool res= Item_func_lpad::fix_length_and_dec();
     maybe_null= true;
     return res;
   }
-  const char *func_name() const { return "lpad_oracle"; }
+  const Schema *schema() const { return &oracle_schema_ref; }
+  void print(String *str, enum_query_type query_type)
+  {
+    if (query_type & QT_FOR_FRM)
+    {
+      // 10.3 downgrade compatibility for FRM
+      str->append(STRING_WITH_LEN("lpad_oracle"));
+    }
+    else
+      print_sql_mode_qualified_name(str, query_type);
+    print_args_parenthesized(str, query_type);
+  }
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_lpad_oracle>(thd, this); }
 };
@@ -1402,6 +1536,9 @@ public:
     collation.set(args[0]->collation);
     ulonglong max_result_length= (ulonglong) args[0]->max_length * 2 +
                                   2 * collation.collation->mbmaxlen;
+    // NULL argument is returned as a string "NULL" without quotes
+    if (args[0]->maybe_null)
+      set_if_bigger(max_result_length, 4 * collation.collation->mbmaxlen);
     max_length= (uint32) MY_MIN(max_result_length, MAX_BLOB_WIDTH);
     return FALSE;
   }
@@ -1457,7 +1594,6 @@ public:
               (cs->mbmaxlen > 1 || !(cs->state & MY_CS_NONASCII))));
     }
   }
-  bool is_json_type() { return args[0]->is_json_type(); }
   String *val_str(String *);
   longlong val_int()
   {
@@ -1677,8 +1813,7 @@ public:
   Item_func_uuid(THD *thd): Item_str_func(thd) {}
   bool fix_length_and_dec()
   {
-    collation.set(system_charset_info,
-                  DERIVATION_COERCIBLE, MY_REPERTOIRE_ASCII);
+    collation.set(DTCollation_numeric());
     fix_char_length(MY_UUID_STRING_LENGTH);
     return FALSE;
   }
@@ -1816,8 +1951,8 @@ public:
   TABLE *table;
   Item_temptable_rowid(TABLE *table_arg);
   const Type_handler *type_handler() const { return &type_handler_string; }
-  Field *create_tmp_field(bool group, TABLE *table)
-  { return create_table_field_from_handler(table); }
+  Field *create_tmp_field(MEM_ROOT *root, bool group, TABLE *table)
+  { return create_table_field_from_handler(root, table); }
   String *val_str(String *str);
   enum Functype functype() const { return  TEMPTABLE_ROWID; }
   const char *func_name() const { return "<rowid>"; }
