@@ -18,10 +18,6 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
@@ -29,7 +25,6 @@
 #include "sql_base.h"
 #include "sql_servers.h"
 #include "tztime.h"
-#endif
 #include "spd_err.h"
 #include "spd_param.h"
 #include "spd_db_include.h"
@@ -44,9 +39,7 @@
 #include "spd_udf.h"
 #include "spd_malloc.h"
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100004
 #define SPIDER_NEED_INIT_ONE_TABLE_FOR_FIND_TEMPORARY_TABLE
-#endif
 
 extern const char **spd_defaults_extra_file;
 extern const char **spd_defaults_file;
@@ -115,21 +108,6 @@ int spider_udf_direct_sql_create_table_list(
     } else
       break;
   }
-#if MYSQL_VERSION_ID < 50500
-  if (!(direct_sql->db_names = (char**)
-    spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_1, MYF(MY_WME | MY_ZEROFILL),
-      &direct_sql->db_names, (uint) (sizeof(char*) * table_count),
-      &direct_sql->table_names, (uint) (sizeof(char*) * table_count),
-      &direct_sql->tables, (uint) (sizeof(TABLE*) * table_count),
-      &tmp_name_ptr, (uint) (sizeof(char) * (
-        table_name_list_length +
-        thd->db_length * table_count +
-        2 * table_count
-      )),
-      &direct_sql->iop, (uint) (sizeof(int) * table_count),
-      NullS))
-  )
-#else
   if (!(direct_sql->db_names = (char**)
     spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_2, MYF(MY_WME | MY_ZEROFILL),
       &direct_sql->db_names, (uint) (sizeof(char*) * table_count),
@@ -146,7 +124,6 @@ int spider_udf_direct_sql_create_table_list(
         (uint) (sizeof(uchar) * ((table_count + 7) / 8)),
       NullS))
   )
-#endif
     DBUG_RETURN(HA_ERR_OUT_OF_MEM);
 
   tmp_ptr = table_name_list;
@@ -413,6 +390,23 @@ int spider_udf_direct_sql_create_conn_key(
   DBUG_RETURN(0);
 }
 
+static inline void spider_maybe_memcpy_string(
+  char **dest,
+  char *src,
+  char *tmp,
+  uint *dest_len,
+  uint src_len)
+{
+    *dest_len= src_len;
+    if (src_len)
+    {
+      *dest= tmp;
+      memcpy(*dest, src, src_len);
+    } else
+      *dest= NULL;
+}
+
+
 SPIDER_CONN *spider_udf_direct_sql_create_conn(
   const SPIDER_DIRECT_SQL *direct_sql,
   int *error_num
@@ -504,89 +498,43 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   {
 #endif
     conn->tgt_port = direct_sql->tgt_port;
-    conn->tgt_socket_length = direct_sql->tgt_socket_length;
-    conn->tgt_socket = tmp_socket;
-    memcpy(conn->tgt_socket, direct_sql->tgt_socket,
-      direct_sql->tgt_socket_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_socket, direct_sql->tgt_socket, tmp_socket,
+      &conn->tgt_socket_length, direct_sql->tgt_socket_length);
     if (!tables_on_different_db_are_joinable)
-    {
-      conn->tgt_db_length = direct_sql->tgt_default_db_name_length;
-      conn->tgt_db = tmp_db;
-      memcpy(conn->tgt_db, direct_sql->tgt_default_db_name,
-        direct_sql->tgt_default_db_name_length);
-    }
-    conn->tgt_username_length = direct_sql->tgt_username_length;
-    conn->tgt_username = tmp_username;
-    memcpy(conn->tgt_username, direct_sql->tgt_username,
-      direct_sql->tgt_username_length);
-    conn->tgt_password_length = direct_sql->tgt_password_length;
-    conn->tgt_password = tmp_password;
-    memcpy(conn->tgt_password, direct_sql->tgt_password,
-      direct_sql->tgt_password_length);
-    conn->tgt_ssl_ca_length = direct_sql->tgt_ssl_ca_length;
-    if (conn->tgt_ssl_ca_length)
-    {
-      conn->tgt_ssl_ca = tmp_ssl_ca;
-      memcpy(conn->tgt_ssl_ca, direct_sql->tgt_ssl_ca,
-        direct_sql->tgt_ssl_ca_length);
-    } else
-      conn->tgt_ssl_ca = NULL;
-    conn->tgt_ssl_capath_length = direct_sql->tgt_ssl_capath_length;
-    if (conn->tgt_ssl_capath_length)
-    {
-      conn->tgt_ssl_capath = tmp_ssl_capath;
-      memcpy(conn->tgt_ssl_capath, direct_sql->tgt_ssl_capath,
-        direct_sql->tgt_ssl_capath_length);
-    } else
-      conn->tgt_ssl_capath = NULL;
-    conn->tgt_ssl_cert_length = direct_sql->tgt_ssl_cert_length;
-    if (conn->tgt_ssl_cert_length)
-    {
-      conn->tgt_ssl_cert = tmp_ssl_cert;
-      memcpy(conn->tgt_ssl_cert, direct_sql->tgt_ssl_cert,
-        direct_sql->tgt_ssl_cert_length);
-    } else
-      conn->tgt_ssl_cert = NULL;
-    conn->tgt_ssl_cipher_length = direct_sql->tgt_ssl_cipher_length;
-    if (conn->tgt_ssl_cipher_length)
-    {
-      conn->tgt_ssl_cipher = tmp_ssl_cipher;
-      memcpy(conn->tgt_ssl_cipher, direct_sql->tgt_ssl_cipher,
-        direct_sql->tgt_ssl_cipher_length);
-    } else
-      conn->tgt_ssl_cipher = NULL;
-    conn->tgt_ssl_key_length = direct_sql->tgt_ssl_key_length;
-    if (conn->tgt_ssl_key_length)
-    {
-      conn->tgt_ssl_key = tmp_ssl_key;
-      memcpy(conn->tgt_ssl_key, direct_sql->tgt_ssl_key,
-        direct_sql->tgt_ssl_key_length);
-    } else
-      conn->tgt_ssl_key = NULL;
-    conn->tgt_default_file_length = direct_sql->tgt_default_file_length;
-    if (conn->tgt_default_file_length)
-    {
-      conn->tgt_default_file = tmp_default_file;
-      memcpy(conn->tgt_default_file, direct_sql->tgt_default_file,
-        direct_sql->tgt_default_file_length);
-    } else
-      conn->tgt_default_file = NULL;
-    conn->tgt_default_group_length = direct_sql->tgt_default_group_length;
-    if (conn->tgt_default_group_length)
-    {
-      conn->tgt_default_group = tmp_default_group;
-      memcpy(conn->tgt_default_group, direct_sql->tgt_default_group,
-        direct_sql->tgt_default_group_length);
-    } else
-      conn->tgt_default_group = NULL;
-    conn->tgt_dsn_length = direct_sql->tgt_dsn_length;
-    if (conn->tgt_dsn_length)
-    {
-      conn->tgt_dsn = tmp_dsn;
-      memcpy(conn->tgt_dsn, direct_sql->tgt_dsn,
-        direct_sql->tgt_dsn_length);
-    } else
-      conn->tgt_dsn = NULL;
+      spider_maybe_memcpy_string(
+        &conn->tgt_db, direct_sql->tgt_default_db_name, tmp_db,
+        &conn->tgt_db_length, direct_sql->tgt_default_db_name_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_username, direct_sql->tgt_username, tmp_username,
+      &conn->tgt_username_length, direct_sql->tgt_username_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_password, direct_sql->tgt_password, tmp_password,
+      &conn->tgt_password_length, direct_sql->tgt_password_length);
+     spider_maybe_memcpy_string(
+      &conn->tgt_ssl_ca, direct_sql->tgt_ssl_ca, tmp_ssl_ca,
+      &conn->tgt_ssl_ca_length, direct_sql->tgt_ssl_ca_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_capath, direct_sql->tgt_ssl_capath, tmp_ssl_capath,
+      &conn->tgt_ssl_capath_length, direct_sql->tgt_ssl_capath_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_cert, direct_sql->tgt_ssl_cert, tmp_ssl_cert,
+      &conn->tgt_ssl_cert_length, direct_sql->tgt_ssl_cert_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_cipher, direct_sql->tgt_ssl_cipher, tmp_ssl_cipher,
+      &conn->tgt_ssl_cipher_length, direct_sql->tgt_ssl_cipher_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_key, direct_sql->tgt_ssl_key, tmp_ssl_key,
+      &conn->tgt_ssl_key_length, direct_sql->tgt_ssl_key_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_default_file, direct_sql->tgt_default_file, tmp_default_file,
+      &conn->tgt_default_file_length, direct_sql->tgt_default_file_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_default_group, direct_sql->tgt_default_group, tmp_default_group,
+      &conn->tgt_default_group_length, direct_sql->tgt_default_group_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_dsn, direct_sql->tgt_dsn, tmp_dsn,
+      &conn->tgt_dsn_length, direct_sql->tgt_dsn_length);
     conn->tgt_ssl_vsc = direct_sql->tgt_ssl_vsc;
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   } else {
@@ -632,12 +580,8 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   }
 #endif
 
-#if MYSQL_VERSION_ID < 50500
-  if (pthread_mutex_init(&conn->mta_conn_mutex, MY_MUTEX_INIT_FAST))
-#else
   if (mysql_mutex_init(spd_key_mutex_mta_conn, &conn->mta_conn_mutex,
     MY_MUTEX_INIT_FAST))
-#endif
   {
     *error_num = HA_ERR_OUT_OF_MEM;
     goto error_mta_conn_mutex_init;
@@ -1139,10 +1083,7 @@ static void spider_minus_1(SPIDER_DIRECT_SQL *direct_sql)
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   direct_sql->access_mode = -1;
 #endif
-#if MYSQL_VERSION_ID < 50500
-#else
   direct_sql->use_real_table = -1;
-#endif
   direct_sql->error_rw_mode = -1;
   for (int i = 0; i < direct_sql->table_count; i++)
     direct_sql->iop[i] = -1;
@@ -1211,10 +1152,7 @@ int spider_udf_parse_direct_sql_param(
         SPIDER_PARAM_STR("srv", server_name);
         SPIDER_PARAM_INT_WITH_MAX("svc", tgt_ssl_vsc, 0, 1);
         SPIDER_PARAM_INT_WITH_MAX("tlm", table_loop_mode, 0, 2);
-#if MYSQL_VERSION_ID < 50500
-#else
         SPIDER_PARAM_INT_WITH_MAX("urt", use_real_table, 0, 1);
-#endif
         SPIDER_PARAM_INT("wto", net_write_timeout, 0);
         error_num= parse.fail(true);
         goto error;
@@ -1265,10 +1203,7 @@ int spider_udf_parse_direct_sql_param(
         error_num= parse.fail(true);
         goto error;
       case 14:
-#if MYSQL_VERSION_ID < 50500
-#else
         SPIDER_PARAM_INT_WITH_MAX("use_real_table", use_real_table, 0, 1);
-#endif
         error_num= parse.fail(true);
         goto error;
       case 15:
@@ -1530,14 +1465,11 @@ void spider_udf_free_direct_sql_alloc(
     pthread_mutex_unlock(direct_sql->bg_mutex);
   }
 #endif
-#if MYSQL_VERSION_ID < 50500
-#else
   if (direct_sql->real_table_used && direct_sql->open_tables_thd)
   {
     spider_sys_close_table(direct_sql->open_tables_thd,
       &direct_sql->open_tables_backup);
   }
-#endif
   if (direct_sql->server_name)
   {
     spider_free(spider_current_trx, direct_sql->server_name, MYF(0));
@@ -1625,11 +1557,8 @@ long long spider_direct_sql_body(
   char *sql;
   TABLE_LIST table_list;
   SPIDER_BG_DIRECT_SQL *bg_direct_sql;
-#if MYSQL_VERSION_ID < 50500
-#else
   TABLE_LIST *real_table_list_last = NULL;
   uint use_real_table = 0;
-#endif
   DBUG_ENTER("spider_direct_sql_body");
   SPIDER_BACKUP_DASTATUS;
   if (!(direct_sql = (SPIDER_DIRECT_SQL *)
@@ -1720,11 +1649,8 @@ long long spider_direct_sql_body(
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   }
 #endif
-#if MYSQL_VERSION_ID < 50500
-#else
   use_real_table = spider_param_udf_ds_use_real_table(thd,
     direct_sql->use_real_table);
-#endif
   for (roop_count = 0; roop_count < direct_sql->table_count; roop_count++)
   {
 #ifdef SPIDER_NEED_INIT_ONE_TABLE_FOR_FIND_TEMPORARY_TABLE
@@ -1755,19 +1681,14 @@ long long spider_direct_sql_body(
     if (!(direct_sql->tables[roop_count] =
       spider_find_temporary_table(thd, &table_list)))
     {
-#if MYSQL_VERSION_ID < 50500
-#else
       if (!use_real_table)
       {
-#endif
         error_num = ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_NUM;
         my_printf_error(ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_NUM,
           ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_STR,
           MYF(0), SPIDER_TABLE_LIST_db_str(&table_list),
           SPIDER_TABLE_LIST_table_name_str(&table_list));
         goto error;
-#if MYSQL_VERSION_ID < 50500
-#else
       }
       TABLE_LIST *tables = &direct_sql->table_list[roop_count];
       MDL_REQUEST_INIT(&tables->mdl_request, MDL_key::TABLE,
@@ -1783,7 +1704,6 @@ long long spider_direct_sql_body(
       real_table_list_last = tables;
       spider_set_bit(direct_sql->real_table_bitmap, roop_count);
       direct_sql->real_table_used = TRUE;
-#endif
     }
   }
   if ((error_num = spider_udf_direct_sql_create_conn_key(direct_sql)))
@@ -1877,6 +1797,11 @@ my_bool spider_direct_sql_init_body(
 ) {
   SPIDER_BG_DIRECT_SQL *bg_direct_sql;
   DBUG_ENTER("spider_direct_sql_init_body");
+  if (!spider_hton_ptr)
+  {
+    strcpy(message, "Plugin 'SPIDER' is not loaded");
+    goto error;
+  }
   if (args->arg_count != 3)
   {
     strcpy(message, "spider_(bg)_direct_sql() requires 3 arguments");
@@ -1900,22 +1825,14 @@ my_bool spider_direct_sql_init_body(
       strcpy(message, "spider_bg_direct_sql() out of memory");
       goto error;
     }
-#if MYSQL_VERSION_ID < 50500
-    if (pthread_mutex_init(&bg_direct_sql->bg_mutex, MY_MUTEX_INIT_FAST))
-#else
     if (mysql_mutex_init(spd_key_mutex_bg_direct_sql,
       &bg_direct_sql->bg_mutex, MY_MUTEX_INIT_FAST))
-#endif
     {
       strcpy(message, "spider_bg_direct_sql() out of memory");
       goto error_mutex_init;
     }
-#if MYSQL_VERSION_ID < 50500
-    if (pthread_cond_init(&bg_direct_sql->bg_cond, NULL))
-#else
     if (mysql_cond_init(spd_key_cond_bg_direct_sql,
       &bg_direct_sql->bg_cond, NULL))
-#endif
     {
       strcpy(message, "spider_bg_direct_sql() out of memory");
       goto error_cond_init;
